package ru.dbotthepony.mc.otm.client.model.entity;

import net.minecraft.client.model.HierarchicalModel;
import net.minecraft.client.model.geom.ModelLayerLocation;
import net.minecraft.client.model.geom.ModelPart;
import net.minecraft.client.model.geom.PartPose;
import net.minecraft.client.model.geom.builders.*;
import net.neoforged.neoforge.client.event.EntityRenderersEvent;
import org.jetbrains.annotations.NotNull;
import ru.dbotthepony.mc.otm.client.animation.BreadMonsterAnimation;
import ru.dbotthepony.mc.otm.client.animation.DroneAnimation;
import ru.dbotthepony.mc.otm.entity.BreadMonster;
import ru.dbotthepony.mc.otm.entity.Drone;
import ru.dbotthepony.mc.otm.registry.MNames;

import static ru.dbotthepony.mc.otm.OverdriveThatMatters.loc;

public class DroneModel {
	public static final ModelLayerLocation LAYER_LOCATION = new ModelLayerLocation(loc(MNames.DRONE), "main");

	private static HierarchicalModel<Drone> model;
	private static LayerDefinition def;

	public static HierarchicalModel<Drone> getModel() {
		if (def == null) {
			def = createBodyLayer();
		}

		if (model == null) {
			return model = new DroneModel.DroneHierarchicalModel<>(def.bakeRoot());
		}


		return model;
	}

	public static LayerDefinition createBodyLayer() {
		MeshDefinition meshdefinition = new MeshDefinition();
		PartDefinition partdefinition = meshdefinition.getRoot();

		PartDefinition root = partdefinition.addOrReplaceChild("root", CubeListBuilder.create(), PartPose.offset(0.0F, 24.0F, 0.0F));

		PartDefinition body = root.addOrReplaceChild("body", CubeListBuilder.create().texOffs(0, 21).addBox(-3.0F, -3.0F, 0.0F, 6.0F, 6.0F, 6.0F, new CubeDeformation(0.0F))
			.texOffs(0, 0).addBox(-6.0F, -6.0F, -8.0F, 12.0F, 12.0F, 9.0F, new CubeDeformation(0.0F))
			.texOffs(24, 21).addBox(-4.0F, -4.0F, -10.0F, 8.0F, 8.0F, 2.0F, new CubeDeformation(0.0F))
			.texOffs(13, 43).addBox(-5.0F, -5.0F, -6.0F, 10.0F, 10.0F, 11.0F, new CubeDeformation(0.0F)), PartPose.offset(0.0F, -8.0F, 1.0F));

		PartDefinition eye = body.addOrReplaceChild("eye", CubeListBuilder.create().texOffs(0, 33).addBox(-3.0F, -3.0F, -3.0F, 6.0F, 6.0F, 6.0F, new CubeDeformation(-0.2F)), PartPose.offsetAndRotation(0.0F, 0.0F, -4.5F, 0.0F, 0.0F, 0.0F));

		PartDefinition gyroL = body.addOrReplaceChild("gyroL", CubeListBuilder.create().texOffs(40, 31).addBox(-1.0F, -3.0F, -3.0F, 2.0F, 6.0F, 6.0F, new CubeDeformation(0.0F)), PartPose.offset(4.0F, 0.0F, 0.0F));

		PartDefinition gyroR = body.addOrReplaceChild("gyroR", CubeListBuilder.create().texOffs(24, 31).addBox(-1.0F, -3.0F, -3.0F, 2.0F, 6.0F, 6.0F, new CubeDeformation(0.0F)), PartPose.offset(-4.0F, 0.0F, 0.0F));

		PartDefinition antennaL = body.addOrReplaceChild("antennaL", CubeListBuilder.create().texOffs(46, -2).addBox(0.0F, -22.0F, -1.0F, 0.0F, 23.0F, 2.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(4.0F, -2.0F, 3.0F, -1.1695F, 0.2013F, 0.0846F));

		PartDefinition antennaR = body.addOrReplaceChild("antennaR", CubeListBuilder.create().texOffs(42, -2).addBox(0.0F, -22.0F, -1.0F, 0.0F, 23.0F, 2.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(-4.0F, -2.0F, 3.0F, -1.1695F, -0.2013F, -0.0846F));

		PartDefinition finL = body.addOrReplaceChild("finL", CubeListBuilder.create().texOffs(39, 0).addBox(-1.0F, 0.0F, -1.0F, 5.0F, 0.0F, 11.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(5.0F, 4.5F, 4.0F, 0.2849F, 0.274F, 0.8249F));

		PartDefinition finR = body.addOrReplaceChild("finR", CubeListBuilder.create().texOffs(39, 11).addBox(-4.0F, 0.0F, -1.0F, 5.0F, 0.0F, 11.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(-5.0F, 4.5F, 4.0F, 0.2849F, -0.274F, -0.8249F));

		model = null;
		return def = LayerDefinition.create(meshdefinition, 64, 64);
	}

	private static class DroneHierarchicalModel<T extends Drone> extends HierarchicalModel<T> {
		private final ModelPart root;
		private final ModelPart body;
		private final ModelPart eye;
		public DroneHierarchicalModel(ModelPart root) {
			this.root = root.getChild("root");
			this.body = this.root.getChild("body");
			this.eye = this.body.getChild("eye");
		}

		@Override
		public void setupAnim(@NotNull T entity, float limbSwing, float limbSwingAmount, float ageInTicks, float netHeadYaw, float headPitch) {
			this.root().getAllParts().forEach(ModelPart::resetPose);

			this.body.getChild("eye").yRot = netHeadYaw * ((float) Math.PI / 180F);
			this.body.getChild("eye").xRot = headPitch * ((float) Math.PI / 180F);

			this.animate(entity.getIdleState(), DroneAnimation.LOOP, ageInTicks, 1.0F);
		}

		@Override
		public @NotNull ModelPart root() {
			return this.root;
		}
	}

	public static void register(EntityRenderersEvent.RegisterLayerDefinitions event) {
		event.registerLayerDefinition(LAYER_LOCATION, DroneModel::createBodyLayer);
	}
}
