package ru.dbotthepony.mc.otm.client.model.entity;

import net.minecraft.client.model.HierarchicalModel;
import net.minecraft.client.model.geom.ModelLayerLocation;
import net.minecraft.client.model.geom.ModelPart;
import net.minecraft.client.model.geom.PartPose;
import net.minecraft.client.model.geom.builders.*;
import net.neoforged.neoforge.client.event.EntityRenderersEvent;
import org.jetbrains.annotations.NotNull;
import ru.dbotthepony.mc.otm.client.animation.EnforcerAnimation;
import ru.dbotthepony.mc.otm.client.animation.LoaderAnimation;
import ru.dbotthepony.mc.otm.entity.Enforcer;
import ru.dbotthepony.mc.otm.registry.MNames;

import static ru.dbotthepony.mc.otm.OverdriveThatMatters.loc;

public class EnforcerModel {
	public static final ModelLayerLocation LAYER_LOCATION = new ModelLayerLocation(loc(MNames.ENFORCER), "main");

	private static HierarchicalModel<Enforcer> model;
	private static LayerDefinition def;

	public static HierarchicalModel<Enforcer> getModel() {
		if (def == null) {
			def = createBodyLayer();
		}

		if (model == null) {
			return model = new EnforcerHierarchicalModel<>(def.bakeRoot());
		}


		return model;
	}

	public static LayerDefinition createBodyLayer() {
		MeshDefinition meshdefinition = new MeshDefinition();
		PartDefinition partdefinition = meshdefinition.getRoot();

		PartDefinition root = partdefinition.addOrReplaceChild("root", CubeListBuilder.create(), PartPose.offset(0.0F, 18.0F, 0.0F));

		PartDefinition base = root.addOrReplaceChild("base", CubeListBuilder.create().texOffs(132, 1).addBox(-8.0F, -2.0F, -7.0F, 16.0F, 5.0F, 14.0F, new CubeDeformation(0.0F))
			.texOffs(92, 0).addBox(-5.0F, 0.0F, -10.0F, 10.0F, 14.0F, 20.0F, new CubeDeformation(0.0F))
			.texOffs(70, 0).addBox(-7.0F, 1.0F, -11.0F, 3.0F, 3.0F, 2.0F, new CubeDeformation(0.0F))
			.texOffs(70, 5).addBox(4.0F, 1.0F, -11.0F, 3.0F, 3.0F, 2.0F, new CubeDeformation(0.0F))
			.texOffs(152, 73).addBox(-6.0F, -5.0F, -6.0F, 12.0F, 3.0F, 12.0F, new CubeDeformation(0.0F)), PartPose.offset(0.0F, -22.0F, 0.0F));

		PartDefinition Body = base.addOrReplaceChild("Body", CubeListBuilder.create().texOffs(70, 0).addBox(-9.0F, -16.4F, -8.1F, 3.0F, 3.0F, 2.0F, new CubeDeformation(0.0F))
			.texOffs(70, 5).addBox(-5.0F, -16.4F, -8.1F, 3.0F, 3.0F, 2.0F, new CubeDeformation(0.0F))
			.texOffs(92, 34).addBox(-11.0F, -15.4F, 10.9F, 22.0F, 14.0F, 14.0F, new CubeDeformation(0.0F))
			.texOffs(0, 0).addBox(-12.0F, -13.4F, -11.1F, 24.0F, 16.0F, 22.0F, new CubeDeformation(0.0F))
			.texOffs(0, 38).addBox(-12.0F, -13.4F, -11.1F, 24.0F, 16.0F, 22.0F, new CubeDeformation(0.5F))
			.texOffs(192, 49).addBox(12.0F, -11.4F, -6.1F, 2.0F, 12.0F, 12.0F, new CubeDeformation(0.0F))
			.texOffs(164, 49).addBox(-14.0F, -11.4F, -6.1F, 2.0F, 12.0F, 12.0F, new CubeDeformation(0.0F))
			.texOffs(92, 62).addBox(-7.0F, 0.0F, 10.0F, 15.0F, 5.0F, 5.0F, new CubeDeformation(0.0F)), PartPose.offset(0.0F, -6.6F, 0.1F));

		PartDefinition cube_r1 = Body.addOrReplaceChild("cube_r1", CubeListBuilder.create().texOffs(80, -2).addBox(0.0F, -18.5F, -1.0F, 0.0F, 19.0F, 2.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(8.0F, -12.9F, 7.9F, -0.1745F, 0.0F, 0.0F));

		PartDefinition Head = Body.addOrReplaceChild("Head", CubeListBuilder.create().texOffs(0, 0).addBox(-4.0F, -1.0F, -1.0F, 7.0F, 3.0F, 3.0F, new CubeDeformation(0.0F)), PartPose.offset(-4.0F, 1.6F, -11.1F));

		PartDefinition rocket = Body.addOrReplaceChild("rocket", CubeListBuilder.create().texOffs(164, 0).addBox(-13.0F, -8.5F, -14.0F, 13.0F, 21.0F, 28.0F, new CubeDeformation(0.0F)), PartPose.offset(-14.0F, -7.9F, -0.1F));

		PartDefinition gun = Body.addOrReplaceChild("gun", CubeListBuilder.create().texOffs(185, 73).addBox(2.0F, -6.0F, -7.0F, 7.0F, 14.0F, 15.0F, new CubeDeformation(0.0F))
			.texOffs(187, 102).addBox(1.0F, -7.0F, -6.0F, 9.0F, 16.0F, 13.0F, new CubeDeformation(-0.6F))
			.texOffs(218, 170).addBox(2.5F, 4.0F, -1.0F, 6.0F, 6.0F, 13.0F, new CubeDeformation(0.0F))
			.texOffs(218, 189).addBox(2.5F, -8.0F, -1.0F, 6.0F, 6.0F, 13.0F, new CubeDeformation(0.0F))
			.texOffs(184, 170).addBox(3.5F, -6.0F, -20.0F, 4.0F, 6.0F, 13.0F, new CubeDeformation(0.0F))
			.texOffs(220, 56).addBox(2.5F, -2.0F, -17.0F, 6.0F, 6.0F, 11.0F, new CubeDeformation(0.0F))
			.texOffs(184, 189).addBox(3.5F, 2.0F, -20.0F, 4.0F, 6.0F, 13.0F, new CubeDeformation(0.0F))
			.texOffs(80, 160).addBox(-6.0F, -5.0F, -5.0F, 10.0F, 10.0F, 10.0F, new CubeDeformation(0.0F)), PartPose.offset(14.0F, -5.5F, 0.0F));

		PartDefinition leg_FL = root.addOrReplaceChild("leg_FL", CubeListBuilder.create().texOffs(45, 76).addBox(-3.561F, -4.2978F, -11.0972F, 7.0F, 9.0F, 13.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(9.5F, -15.5F, -5.5F, 0.9425F, -0.5061F, -0.0349F));

		PartDefinition joint0_FL = leg_FL.addOrReplaceChild("joint0_FL", CubeListBuilder.create().texOffs(82, 233).addBox(-3.061F, -9.3731F, -7.1617F, 6.0F, 15.0F, 8.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(0.0F, -3.0F, -8.0F, -0.3054F, 0.0F, 0.0F));

		PartDefinition joint1_FL = joint0_FL.addOrReplaceChild("joint1_FL", CubeListBuilder.create().texOffs(88, 91).addBox(-2.061F, -1.7726F, -14.7213F, 4.0F, 3.0F, 16.0F, new CubeDeformation(0.0F))
			.texOffs(110, 230).addBox(-4.061F, -8.7726F, -15.7213F, 8.0F, 7.0F, 19.0F, new CubeDeformation(0.0F))
			.texOffs(103, 225).addBox(-3.061F, -4.7726F, -15.7213F, 6.0F, 9.0F, 6.0F, new CubeDeformation(0.0F))
			.texOffs(30, 124).addBox(-5.061F, -2.7726F, -25.7213F, 2.0F, 5.0F, 13.0F, new CubeDeformation(0.0F))
			.texOffs(30, 142).addBox(2.939F, -1.7726F, -25.7213F, 2.0F, 5.0F, 13.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(0.0F, -9.0F, -4.0F, 0.1309F, 0.0F, 0.0F));

		PartDefinition cube_r2 = joint1_FL.addOrReplaceChild("cube_r2", CubeListBuilder.create().texOffs(24, 184).addBox(-3.939F, 1.6269F, -0.1617F, 7.0F, 10.0F, 5.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(0.5F, -6.5F, -12.0F, -1.9635F, 0.0F, 0.0F));

		PartDefinition wheelFL = joint1_FL.addOrReplaceChild("wheelFL", CubeListBuilder.create(), PartPose.offset(-0.061F, 0.2274F, -23.7213F));

		PartDefinition cube_r3 = wheelFL.addOrReplaceChild("cube_r3", CubeListBuilder.create().texOffs(184, 232).addBox(-3.061F, -5.9313F, -7.9979F, 6.0F, 12.0F, 12.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(0.061F, -1.4613F, 1.3641F, 0.7854F, 0.0F, 0.0F));

		PartDefinition leg_FR = root.addOrReplaceChild("leg_FR", CubeListBuilder.create().texOffs(0, 76).addBox(-3.439F, -4.2978F, -11.0972F, 7.0F, 9.0F, 13.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(-9.5F, -15.5F, -5.5F, 0.9425F, 0.5061F, 0.0349F));

		PartDefinition joint0_FR = leg_FR.addOrReplaceChild("joint0_FR", CubeListBuilder.create().texOffs(0, 233).addBox(-2.939F, -9.3731F, -7.1617F, 6.0F, 15.0F, 8.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(0.0F, -3.0F, -8.0F, -0.3054F, 0.0F, 0.0F));

		PartDefinition joint1_FR = joint0_FR.addOrReplaceChild("joint1_FR", CubeListBuilder.create().texOffs(88, 72).addBox(-1.939F, -1.7726F, -14.7213F, 4.0F, 3.0F, 16.0F, new CubeDeformation(0.0F))
			.texOffs(28, 230).addBox(-3.939F, -8.7726F, -15.7213F, 8.0F, 7.0F, 19.0F, new CubeDeformation(0.0F))
			.texOffs(21, 225).addBox(-2.939F, -4.7726F, -15.7213F, 6.0F, 9.0F, 6.0F, new CubeDeformation(0.0F))
			.texOffs(0, 124).addBox(-4.939F, -2.7726F, -25.7213F, 2.0F, 5.0F, 13.0F, new CubeDeformation(0.0F))
			.texOffs(0, 142).addBox(3.061F, -2.7726F, -25.7213F, 2.0F, 5.0F, 13.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(0.0F, -9.0F, -4.0F, 0.1309F, 0.0F, 0.0F));

		PartDefinition cube_r4 = joint1_FR.addOrReplaceChild("cube_r4", CubeListBuilder.create().texOffs(24, 184).addBox(-3.939F, 1.6269F, -0.1617F, 7.0F, 10.0F, 5.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(0.5F, -6.5F, -12.0F, -1.9635F, 0.0F, 0.0F));

		PartDefinition wheelFR = joint1_FR.addOrReplaceChild("wheelFR", CubeListBuilder.create(), PartPose.offset(0.061F, 0.2274F, -23.7213F));

		PartDefinition cube_r5 = wheelFR.addOrReplaceChild("cube_r5", CubeListBuilder.create().texOffs(220, 232).addBox(-2.939F, -5.9313F, -7.9979F, 6.0F, 12.0F, 12.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(-0.061F, -1.4613F, 1.3641F, 0.7854F, 0.0F, 0.0F));

		PartDefinition leg_BL = root.addOrReplaceChild("leg_BL", CubeListBuilder.create().texOffs(45, 100).addBox(-3.561F, -3.9644F, -2.2362F, 7.0F, 9.0F, 13.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(9.5F, -15.8333F, 5.8333F, -0.9425F, 0.5061F, -0.0349F));

		PartDefinition joint0_BL = leg_BL.addOrReplaceChild("joint0_BL", CubeListBuilder.create().texOffs(0, 202).addBox(-3.061F, -9.0749F, -0.7948F, 6.0F, 15.0F, 8.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(0.0F, -2.6667F, 7.6667F, 0.3491F, 0.0F, 0.0F));

		PartDefinition joint1_BL = joint0_BL.addOrReplaceChild("joint1_BL", CubeListBuilder.create().texOffs(128, 72).addBox(-1.939F, -1.7726F, -14.7213F, 4.0F, 3.0F, 16.0F, new CubeDeformation(0.0F))
			.texOffs(28, 199).addBox(-3.939F, -8.7726F, -15.7213F, 8.0F, 7.0F, 19.0F, new CubeDeformation(0.0F))
			.texOffs(0, 187).addBox(-2.939F, -4.8726F, -15.7213F, 6.0F, 9.0F, 6.0F, new CubeDeformation(0.0F))
			.texOffs(60, 124).addBox(-4.939F, -2.7726F, -25.7213F, 2.0F, 5.0F, 13.0F, new CubeDeformation(0.0F))
			.texOffs(60, 142).addBox(3.061F, -2.8726F, -25.7213F, 2.0F, 5.0F, 13.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(0.0F, -8.7509F, 4.1156F, -2.9671F, 0.0F, -3.1416F));

		PartDefinition thruster_r1 = joint1_BL.addOrReplaceChild("thruster_r1", CubeListBuilder.create().texOffs(0, 160).addBox(0.0F, -9.0F, -14.0F, 7.0F, 11.0F, 13.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(-3.5F, -2.7491F, -3.6156F, -0.7418F, 0.0F, 0.0F));

		PartDefinition cube_r6 = joint1_BL.addOrReplaceChild("cube_r6", CubeListBuilder.create().texOffs(48, 184).addBox(-3.939F, 1.6269F, -0.1617F, 7.0F, 10.0F, 5.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(0.5F, -6.7491F, -26.1156F, 1.1781F, 0.0F, 0.0F));

		PartDefinition wheelBL = joint1_BL.addOrReplaceChild("wheelBL", CubeListBuilder.create(), PartPose.offset(0.061F, 0.2274F, -22.7213F));

		PartDefinition cube_r7 = wheelBL.addOrReplaceChild("cube_r7", CubeListBuilder.create().texOffs(184, 208).addBox(-2.939F, -5.9313F, -7.9979F, 6.0F, 12.0F, 12.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(-0.061F, -1.4613F, 1.3641F, 0.7854F, 0.0F, 0.0F));

		PartDefinition leg_BR = root.addOrReplaceChild("leg_BR", CubeListBuilder.create().texOffs(1, 100).addBox(-3.439F, -3.9644F, -1.5695F, 7.0F, 9.0F, 13.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(-9.5F, -15.8333F, 5.1667F, -0.9425F, -0.5061F, 0.0349F));

		PartDefinition joint0_BR = leg_BR.addOrReplaceChild("joint0_BR", CubeListBuilder.create().texOffs(82, 202).addBox(-2.939F, -9.0749F, -0.7948F, 6.0F, 15.0F, 8.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(0.0F, -2.6667F, 8.3333F, 0.3491F, 0.0F, 0.0F));

		PartDefinition joint1_BR = joint0_BR.addOrReplaceChild("joint1_BR", CubeListBuilder.create().texOffs(128, 91).addBox(-2.061F, -1.7726F, -14.7213F, 4.0F, 3.0F, 16.0F, new CubeDeformation(0.0F))
			.texOffs(110, 199).addBox(-4.061F, -8.7726F, -15.7213F, 8.0F, 7.0F, 19.0F, new CubeDeformation(0.0F))
			.texOffs(83, 187).addBox(-3.061F, -4.7726F, -15.7213F, 6.0F, 9.0F, 6.0F, new CubeDeformation(0.0F))
			.texOffs(90, 124).addBox(-5.061F, -2.7726F, -25.7213F, 2.0F, 5.0F, 13.0F, new CubeDeformation(0.0F))
			.texOffs(90, 142).addBox(2.939F, -2.7726F, -25.7213F, 2.0F, 5.0F, 13.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(0.0F, -8.7509F, 4.1156F, -2.9671F, 0.0F, -3.1416F));

		PartDefinition thruster_r2 = joint1_BR.addOrReplaceChild("thruster_r2", CubeListBuilder.create().texOffs(40, 160).addBox(0.0F, -9.0F, -14.0F, 7.0F, 11.0F, 13.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(-3.5F, -2.7491F, -3.6156F, -0.7418F, 0.0F, 0.0F));

		PartDefinition cube_r8 = joint1_BR.addOrReplaceChild("cube_r8", CubeListBuilder.create().texOffs(48, 184).addBox(-3.939F, 1.6269F, -0.1617F, 7.0F, 10.0F, 5.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(0.5F, -6.7491F, -26.1156F, 1.1781F, 0.0F, 0.0F));

		PartDefinition wheelBR = joint1_BR.addOrReplaceChild("wheelBR", CubeListBuilder.create(), PartPose.offset(-0.061F, 0.2274F, -23.7213F));

		PartDefinition cube_r9 = wheelBR.addOrReplaceChild("cube_r9", CubeListBuilder.create().texOffs(220, 208).addBox(-3.061F, -5.9313F, -7.9979F, 6.0F, 12.0F, 12.0F, new CubeDeformation(0.0F)), PartPose.offsetAndRotation(0.061F, -1.4613F, 1.3641F, 0.7854F, 0.0F, 0.0F));


		model = null;
		return def = LayerDefinition.create(meshdefinition, 256, 256);
	}

	private static class EnforcerHierarchicalModel<T extends Enforcer> extends HierarchicalModel<T> {
		private final ModelPart root;
		private final ModelPart base;
		private final ModelPart Body;
		private final ModelPart Head;
		private final ModelPart rocket;
		private final ModelPart gun;
		public EnforcerHierarchicalModel(ModelPart root) {
			this.root = root.getChild("root");
			this.base = this.root.getChild("base");
			this.Body = this.base.getChild("Body");
			this.Head = this.Body.getChild("Head");
			this.rocket = this.Body.getChild("rocket");
			this.gun = this.Body.getChild("gun");
		}

		@Override
		public void setupAnim(@NotNull T entity, float limbSwing, float limbSwingAmount, float ageInTicks, float netHeadYaw, float headPitch) {
			this.root().getAllParts().forEach(ModelPart::resetPose);

			this.Head.xRot = headPitch * ((float) Math.PI / 180F);
			this.Body.yRot = netHeadYaw * ((float) Math.PI / 180F);

			this.rocket.xRot = headPitch * ((float) Math.PI / 180F);
			this.gun.xRot = headPitch * ((float) Math.PI / 180F);


			//this.animate(entity.getChargeState(), EnforcerAnimation.CHARGE, ageInTicks, 1.0F);
			this.animateWalk(EnforcerAnimation.CHARGE, limbSwing, limbSwingAmount, 0.8F, 2.5F);
			this.animate(entity.getIdleState(), EnforcerAnimation.IDLE, ageInTicks, 1.0F);

		}

		public ModelPart getHead() {
			return this.Body.getChild("Head");
		}


		@Override
		public @NotNull ModelPart root() {
			return this.root;
		}
	}

	public static void register(EntityRenderersEvent.RegisterLayerDefinitions event) {
		event.registerLayerDefinition(LAYER_LOCATION, EnforcerModel::createBodyLayer);
	}
}
