package ru.dbotthepony.mc.otm.capability.matter

import net.minecraft.ChatFormatting
import net.minecraft.network.chat.Component
import net.minecraft.world.item.Item
import net.minecraft.world.item.Item.TooltipContext
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.TooltipFlag
import ru.dbotthepony.kommons.math.RGBAColor
import ru.dbotthepony.mc.otm.util.TranslatableComponent
import java.util.*
import java.util.function.Predicate
import java.util.stream.Collectors
import java.util.stream.Stream
import kotlin.math.roundToInt

interface IPatternStorage {
	/**
	 * Patterns stored in this pattern storage
	 */
	val patterns: Stream<PatternState>

	fun findPatterns(item: Item): Collection<PatternState> {
		return findPatterns { item == it.item }
	}

	fun findPatterns(predicate: Predicate<PatternState>): Collection<PatternState> {
		return patterns.filter(predicate).collect(Collectors.toList())
	}

	fun findPattern(item: Item): PatternState? {
		return patterns.filter { it.item == item }.findAny().orElse(null)
	}

	fun getPattern(id: UUID?): PatternState? {
		return patterns.filter { it.id == id }.findAny().orElse(null)
	}

	fun hasPattern(item: Item): Boolean {
		return patterns.filter { it.item == item }.findAny().isPresent
	}

	fun hasPattern(state: PatternState): Boolean {
		return hasPattern(state.id)
	}

	fun hasPattern(id: UUID?): Boolean {
		return patterns.filter { it.id == id }.findAny().isPresent
	}

	val patternCapacity: Int
	val storedPatterns: Int

	/**
	 * Prefer way to call it is to first call only_update = true
	 * and if it fail, try only_update = false
	 *
	 * @param pattern pattern to be inserted or update value from
	 * @param onlyUpdate do not insert new pattern if this pattern's UUID is not matched
	 * @param simulate whenever to affect state
	 * @return record of status of the operation (at status() FAIL, UPDATED, INSERTED) as well as new_state and old_state
	 */
	fun insertPattern(pattern: PatternState, onlyUpdate: Boolean, simulate: Boolean): PatternInsertStatus

	fun insertPattern(pattern: PatternState, simulate: Boolean): PatternInsertStatus {
		return insertPattern(pattern, false, simulate)
	}

	fun updatePattern(pattern: PatternState, simulate: Boolean): PatternInsertStatus {
		return insertPattern(pattern, true, simulate)
	}
}

fun IPatternStorage.getBarWidth(): Int {
	return ((storedPatterns.toFloat() / patternCapacity.toFloat()).coerceAtLeast(0f).coerceAtMost(1f) * 13f).roundToInt()
}

fun IPatternStorage.getBarColor(): Int {
	return RGBAColor.LOW_PATTERNS.linearInterpolation((storedPatterns / patternCapacity).toFloat(), RGBAColor.FULL_PATTERNS).toBGR()
}

fun IPatternStorage.gatherTooltip(
	context: TooltipContext,
	components: MutableList<Component>,
	tooltipType: TooltipFlag
) {
	for (state in patterns) {
		components.add(
			TranslatableComponent(
				"otm.item.pattern.line",
				state.item.getName(ItemStack(state.item, 1)),
				String.format("%.2f", state.researchPercent * 100.0)
			).withStyle(ChatFormatting.AQUA)
		)
	}
}
