package ru.dbotthepony.mc.otm.client.renderer.entity

import com.mojang.blaze3d.vertex.PoseStack
import net.minecraft.client.model.geom.ModelPart
import net.minecraft.client.renderer.MultiBufferSource
import net.minecraft.client.renderer.RenderType
import net.minecraft.client.renderer.entity.EntityRenderer
import net.minecraft.client.renderer.entity.EntityRendererProvider
import net.minecraft.client.renderer.texture.OverlayTexture
import net.minecraft.resources.ResourceLocation
import net.minecraft.world.phys.Vec3
import ru.dbotthepony.mc.otm.OverdriveThatMatters.loc
import ru.dbotthepony.mc.otm.client.model.entity.RocketModel
import ru.dbotthepony.mc.otm.entity.RocketProjectile

class RocketRenderer(context: EntityRendererProvider.Context) : EntityRenderer<RocketProjectile>(context) {

	private val model: ModelPart = RocketModel.createBodyLayer().bakeRoot()
	private val TEXTURE = loc("textures/entity/missile.png")

	override fun getTextureLocation(entity: RocketProjectile): ResourceLocation = TEXTURE

	override fun render(
		entity: RocketProjectile,
		yaw: Float,
		partialTicks: Float,
		poseStack: PoseStack,
		buffer: MultiBufferSource,
		packedLight: Int
	) {
		poseStack.pushPose()

		val motion: Vec3 = entity.deltaMovement
		val speed = motion.length()

		if (speed > 0) {
			val yawRotation = Math.toDegrees(Math.atan2(motion.x, motion.z)).toFloat()

			val pitch = Math.toDegrees(-motion.y / speed).toFloat()

			poseStack.mulPose(com.mojang.math.Axis.YP.rotationDegrees(yawRotation))
			poseStack.mulPose(com.mojang.math.Axis.XP.rotationDegrees(pitch+ 180F))
		}

		val vertexConsumer = buffer.getBuffer(RenderType.entityCutout(TEXTURE))
		model.render(poseStack, vertexConsumer, 15728880, OverlayTexture.NO_OVERLAY)

		poseStack.popPose()
		super.render(entity, yaw, partialTicks, poseStack, buffer, packedLight)
	}
}
