package ru.dbotthepony.mc.otm.container.slotted

import net.minecraft.core.HolderLookup
import net.minecraft.nbt.CompoundTag
import net.minecraft.nbt.NbtOps
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import org.apache.logging.log4j.LogManager
import ru.dbotthepony.mc.otm.container.IFilteredAutomatedContainerSlot
import ru.dbotthepony.mc.otm.container.ItemFilter
import ru.dbotthepony.mc.otm.util.set

open class FilteredContainerSlot(
	container: SlottedContainer,
	slot: Int
) : ContainerSlot(container, slot), IFilteredAutomatedContainerSlot {
	override var filter: ItemFilter = ItemFilter.EMPTY
		set(value) {
			if (field !== value) {
				field = value
				container.notifyChanged()
			}
		}

	override fun clear() {
		super.clear()
		filter = ItemFilter.EMPTY
	}

	override fun serializeNBT(provider: HolderLookup.Provider): CompoundTag {
		return super.serializeNBT(provider).also {
			it["filter"] = ItemFilter.CODEC.encodeStart(provider.createSerializationContext(NbtOps.INSTANCE), filter)
				.getOrThrow { RuntimeException("Failed to serialize item filter: $it") }
		}
	}

	override fun deserializeNBT(provider: HolderLookup.Provider, nbt: CompoundTag) {
		super.deserializeNBT(provider, nbt)

		filter = ItemFilter.EMPTY

		if ("filter" in nbt) {
			ItemFilter.CODEC.decode(provider.createSerializationContext(NbtOps.INSTANCE), nbt["filter"])
				.ifError { LOGGER.error("Unable to deserialize item filter: ${it.message()}") }
				.resultOrPartial().map { it.first }.ifPresent { filter = it }
		}
	}

	class Simple(
		private val listener: (new: ItemStack, old: ItemStack) -> Unit = { _, _ -> },
		private val maxStackSize: Int = Item.DEFAULT_MAX_STACK_SIZE,
		private val canAutomationPlaceItem: AutomationPlaceItem<FilteredContainerSlot> = AutomationPlaceItem { _, _ -> true },
		private val canAutomationTakeItem: AutomationTakeItem<FilteredContainerSlot> = AutomationTakeItem { _, _ -> true },
		private val modifyAutomationPlaceCount: AutomationModifyPlaceCount<FilteredContainerSlot> = AutomationModifyPlaceCount { _, item -> item.count },
		private val modifyAutomationExtractionCount: AutomationModifyExtractionCount<FilteredContainerSlot> = AutomationModifyExtractionCount { _, desired -> desired },
	) : SlottedContainer.SlotProvider<FilteredContainerSlot> {
		constructor(
			listener: (new: ItemStack, old: ItemStack) -> Unit = { _, _ -> },
			maxStackSize: Int = Item.DEFAULT_MAX_STACK_SIZE,
			filter: AutomationFilter<FilteredContainerSlot> = AutomationFilters.ALLOW
		) : this(listener, maxStackSize, filter, filter, filter, filter)

		private open inner class Instance(container: SlottedContainer, slot: Int) : FilteredContainerSlot(container, slot) {
			override val maxStackSize: Int
				get() = this@Simple.maxStackSize

			override fun maxStackSize(item: ItemStack): Int {
				if (maxStackSize == Int.MAX_VALUE)
					return Int.MAX_VALUE

				return super.maxStackSize(item)
			}

			override fun notifyChanged(old: ItemStack) {
				super.notifyChanged(old)
				listener(item, old)
			}

			override fun canAutomationPlaceItem(itemStack: ItemStack): Boolean {
				return super.canAutomationPlaceItem(itemStack) && canAutomationPlaceItem.canAutomationPlaceItem(this, itemStack)
			}

			override fun canAutomationTakeItem(desired: Int): Boolean {
				return super.canAutomationTakeItem(desired) && canAutomationTakeItem.canAutomationTakeItem(this, desired)
			}

			override fun modifyAutomationPlaceCount(itemStack: ItemStack): Int {
				return modifyAutomationPlaceCount.modifyAutomationPlaceCount(this, itemStack)
			}

			override fun modifyAutomationExtractionCount(desired: Int): Int {
				return modifyAutomationExtractionCount.modifyAutomationExtractionCount(this, desired)
			}
		}

		override fun create(container: SlottedContainer, index: Int): FilteredContainerSlot {
			return Instance(container, index)
		}
	}

	companion object {
		private val LOGGER = LogManager.getLogger()
	}
}
